<?php

namespace Porth\Borderstyle\Controller;


    /***************************************************************
     *
     *  Copyright notice
     *
     *  (c) 2016 Dr. Dieter Porth <info@mobger.de>
     *
     *  All rights reserved
     *
     *  This script is free software; you can redistribute it and/or modify
     *  it under the terms of the GNU General Public License as published by
     *  the Free Software Foundation; either version 3 of the License, or
     *  (at your option) any later version.
     *
     *  The GNU General Public License can be found at
     *  http://www.gnu.org/copyleft/gpl.html.
     *
     *  This copyright notice MUST APPEAR in all copies of the script!
     ***************************************************************/

/**
 * ThemeController
 */
class ThemeController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    const SELFJSON_MARKER_VALUE = '###value###';

    /**
     * themeRepository
     *
     * @var \Porth\Borderstyle\Domain\Repository\ThemeRepository
     * @inject
     */
    protected $themeRepository = NULL;


    /**
     * @param int $borderstyleUid
     * @return null|\Porth\Borderstyle\Domain\Model\Theme
     */
    protected function getBorderStyle($borderstyleUid)
    {
        $uid = (int)$borderstyleUid;
        $uid = (!$uid ? (int)$this->settings['flexForm']['borderstyleUid'] : $uid);
        $uid = (!$uid ? (int)$this->settings['borderstyleUid'] : $uid);

        /** @var \Porth\Borderstyle\Domain\Model\Theme $borderstyle */
        $borderstyle = null;
        if ($uid > 0) {
            $borderstyle = $this->themeRepository->findByUid($uid);
        }
        return $borderstyle;
    }

    /**
     * find all allowed borderstyles
     *
     * @param string $storagePid
     * @return array|\TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    protected function getAllBorderStyle($storagePid)
    {
        $pids = trim($storagePid);
        $pids = (((!$pids) && ($pids !== '0') && isset($this->settings['flexForm']['storagePid'])) ?
            trim($this->settings['flexForm']['storagePid']) :
            $pids);
        $pids = (((!$pids) && ($pids !== '0') && isset($this->settings['storagePid'])) ?
            trim($this->settings['storagePid']) :
            $pids);

        $storagePidAry = (((!$pids) || ($pids === '0')) ?
            [] :
            array_map('intval', explode(',', $pids)));
        $borderstyles = $this->themeRepository->findAllOptions($storagePidAry);
        return $borderstyles;
    }

    /**
     * fill a non-presistence field with the value
     *
     * @param $borderstyles
     * @return mixed
     */
    protected function addPrefixToClassname($borderstyles ) {
        /** @var \Porth\Borderstyle\Domain\Model\Theme $borderstyle */
        foreach ($borderstyles as $borderstyle) {
            $newClassName = $this->settings['vendorClass'] . $borderstyle->getClassName();
            $borderstyle->setPrefixClassName($newClassName);
        }
        return $borderstyles;
    }

    /**
     * action classname for an injection of the classname
     * (It should be easier, to do it directly.)
     *
     * @param int $borderstyleUid
     * @param array $params
     * @return void
     */
    public function classnameAction($borderstyleUid = 0, $params = [])
    {
        $borderstyle = self::getBorderStyle($borderstyleUid);
        $this->view->assign('borderstyle', $borderstyle);
        $this->view->assign('params', $params);

    }

    /**
     * action classnameoptions generates al ist of Option-Field, which could integrated externaly with other optionlists in a select-field
     *
     * @param array $params - not used here, but part of my communication-convention between extensions in TYPO3, cann include the values of storagePids
     */
    public function classnameoptionsAction($params = [])
    {
        $storagePid = (
            isset($params['storagePid']) ?
            $params['storagePid'] :
            ''
        );

        $borderstyles = self::getAllBorderStyle($storagePid);
        $borderstyles = self::addPrefixToClassname($borderstyles);

        $this->view->assign('borderstyles', $borderstyles);
        $this->view->assign('storagePid', $storagePid);
        $this->view->assign('params', $params);

    }

    /**
     * action classnameoptions generate a select-field with the prefixed class-name as valus
     *
     * @param string $name
     * @param array $params - not used here, but part of my communication-convention between extensions in TYPO3, cann include the values of storagePids
     */
    public function classnameselectAction($name, $params = [])
    {
        self::classnameoptionsAction($params);
        $this->view->assign('name', $name);
    }

    /**
     * action generate the default-styles an write them to typoscript, which put it into the default-css of TYPO3
     *
     * @param array $params - not used here, but part of my communication-convention between extensions in TYPO3
     */
    public function stylesheetsAction($params = [])
    {
        $storagePid = (
            isset($params['storagePid']) ?
            $params['storagePid'] :
            ''
        );

        $borderstyles = self::getAllBorderStyle($storagePid);
        $borderstyles = self::addPrefixToClassname($borderstyles);

        $this->view->assign('borderstyles', $borderstyles);
        $this->view->assign('params', $params);
    }

    /**
     * action options to list allowed uids with names in a select-box
     *
     * @param string $storagePid - comaseparated List of pids or empty
     * @param array $params - not used here, but part of the communication-protocol between extensions in TYPO3
     * @return void
     */
    public function interntestAction($storagePid = '', $params = [])
    {
        $borderstyles = self::getAllBorderStyle($storagePid);
        $borderstyles = self::addPrefixToClassname($borderstyles);
        $this->view->assign('borderstyles', $borderstyles);
        $this->view->assign('params', $params);

    }

    /**
     * action with tests for external extensions
     *
     * @param string $storagePid - comaseparated List of pids or empty
     * @param array $params - not used here, but part of the communication-protocol between extensions in TYPO3
     * @return void
     */
    public function testingAction($storagePid = '', $params = [])
    {
        $borderstyles = self::getAllBorderStyle($storagePid);
        $borderstyles = self::addPrefixToClassname($borderstyles);
        $this->view->assign('borderstyles', $borderstyles);
        $this->view->assign('params', $params);
    }

}
